package net.kldp.beat.system.interceptor;

import java.lang.annotation.Annotation;
import java.util.HashMap;

import net.kldp.beat.exception.fatal.InterceptorException;
import net.kldp.beat.interceptor.SystemInterceptor;
import net.kldp.beat.system.annotation.CookieMap;
import net.kldp.beat.system.annotation.HeaderMap;
import net.kldp.beat.system.annotation.MultipartRequest;
import net.kldp.beat.system.annotation.Parameter;
import net.kldp.beat.system.annotation.ParameterMap;
import net.kldp.beat.system.annotation.RequestMap;
import net.kldp.beat.system.annotation.ServletContext;
import net.kldp.beat.system.annotation.ServletDispatcher;
import net.kldp.beat.system.annotation.ServletOutputStream;
import net.kldp.beat.system.annotation.ServletRequest;
import net.kldp.beat.system.annotation.ServletResponse;
import net.kldp.beat.system.annotation.SessionMap;
import net.kldp.beat.system.annotation.ApplicationMap;

/**
 * 인터셉터의 인스턴스를 생성하고 리턴하는 팩터리 클래스입니다.
 * 
 */
public final class SystemInterceptorPool {
	private static HashMap<Class<? extends Annotation>, SystemInterceptor> interceptors = new HashMap<Class<? extends Annotation>, SystemInterceptor>();
	static {
		interceptors.put(ApplicationMap.class, new ApplicationMapInterceptor());
		interceptors.put(Parameter.class, new ParameterInterceptor());
		interceptors.put(ParameterMap.class, new ParameterMapInterceptor());
		interceptors.put(MultipartRequest.class, new MultipartInterceptor());
		interceptors.put(RequestMap.class, new RequestMapInterceptor());
		interceptors.put(ServletContext.class, new ServletContextInterceptor());
		interceptors.put(ServletDispatcher.class, new ServletDispatcherInterceptor());
		interceptors.put(ServletRequest.class, new ServletRequestInterceptor());
		interceptors.put(ServletResponse.class, new ServletResponseInterceptor());
		interceptors.put(ServletOutputStream.class, new ServletOutputStreamInterceptor());
		interceptors.put(HeaderMap.class, new HeaderMapInterceptor());
		interceptors.put(CookieMap.class, new CookieMapInterceptor());
		interceptors.put(SessionMap.class, new SessionMapInterceptor());
	}

	private SystemInterceptorPool() {
	}

	/**
	 * 어노테이션에 맞는 시스템 인터셉터를 리턴합니다. 시스템 어노테이션의 경우 싱글턴으로 생성되어 리턴됩니다.
	 * 
	 * @param annotation
	 * @return Interceptor
	 * @throws InterceptorException
	 */
	public static SystemInterceptor getInterceptor(Annotation annotation) throws InterceptorException {
		Class<? extends Annotation> clazz = annotation.annotationType();
		if (interceptors.containsKey(clazz))
			return interceptors.get(clazz);
		throw new InterceptorException("Can not found System Interceptor.");
	}
}